package com.chinaitop.depot.system.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.support.ExcelTypeEnum;
import com.chinaitop.depot.system.mapper.UserOperationPageLogMapper;
import com.chinaitop.depot.system.model.UserOperationPageLog;
import com.chinaitop.depot.system.model.UserOperationPageLogExample;
import com.chinaitop.depot.system.model.UserOperationPageLogVO;
import com.chinaitop.depot.system.param.UserOperationPageLogParam;
import com.chinaitop.depot.system.service.UserOperationPageLogService;
import com.chinaitop.depot.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @author qingsong.han
 * @description:
 * @create 2024-11-04 10:57
 */
@Slf4j
@Service
public class UserOperationPageLogServiceImpl implements UserOperationPageLogService {
    @Resource
    private UserOperationPageLogMapper pageLogMapper;

    @Override
    public List<UserOperationPageLog> getPageList(UserOperationPageLogParam param) {
        UserOperationPageLogExample example = new UserOperationPageLogExample();
        UserOperationPageLogExample.Criteria criteria = example.createCriteria();
        if (StringUtils.hasText(param.getViewName())) {
            criteria.andViewNameLike(param.getViewName());
        }
        if (StringUtils.hasText(param.getRealName())) {
            criteria.andRealNameLike(param.getRealName());
        }
        if (!ObjectUtils.isEmpty(param.getVisitPageTimeStart()) && !ObjectUtils.isEmpty(param.getVisitPageTimeEnd())) {
            criteria.andVisitPageTimeBetween(param.getVisitPageTimeStart(), param.getVisitPageTimeEnd());
        }
        return pageLogMapper.selectByExample(example);
    }

    @Override
    public void export(String fileName, UserOperationPageLogParam param, HttpServletRequest request, HttpServletResponse response) {
        try {
            if (!StringUtils.hasText(fileName)) {
                fileName = LocalDateTime.now().toString();
            }

            // 设置响应信息，让浏览器下载文件
            fileName = URLEncoder.encode(fileName, "UTF-8").replaceAll("\\+", "%20");
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");

            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");

            EasyExcel.write(response.getOutputStream()) // 或, GrainTempExcel.class
                    .head(UserOperationPageLogVO.class) // write 中, GrainTempExcel.class 二选一
                    .excelType(ExcelTypeEnum.XLSX)
                    .sheet("用户访问日志")
                    .doWrite(this.getData(param));

        } catch (IOException e) {
            log.error("摄像头管理导出异常：{}", e);
            throw new RuntimeException(e);
        }
    }

    private List<UserOperationPageLogVO> getData(UserOperationPageLogParam param) {
        List<UserOperationPageLog> pageList = this.getPageList(param);
        List<UserOperationPageLogVO> list = new ArrayList<>();
        pageList.forEach(userLoginLog -> {
            UserOperationPageLogVO logVO = new UserOperationPageLogVO();
            BeanUtils.copyProperties(userLoginLog, logVO);
            list.add(logVO);
        });
        return list;
    }
}
